// --------------------------------------------------------------------------
// Licensed Materials - Property of IBM
//
// 5725-A06 5725-A29 5724-Y48 5724-Y49 5724-Y54 5724-Y55
// Copyright IBM Corporation 1998, 2013. All Rights Reserved.
//
// Note to U.S. Government Users Restricted Rights:
// Use, duplication or disclosure restricted by GSA ADP Schedule
// Contract with IBM Corp.
// --------------------------------------------------------------------------
// October 13, 2015. This is the model file used for basic MILP.
//

int   NbProducts = ...;
int   NbRetailers = ...;

range Products = 1..NbProducts;
range Retailers =1..NbRetailers;

int Capacity = ...;
float NoPurchase[Retailers] = ...;
float Fraction[Retailers] = ...;
float Revenue[Retailers][Products]=...;
float Preference[Retailers][Products] = ...;
float Weight[Products]=...;
float MaximumCap[Retailers][Products]=...;
float RevenueMax[Retailers]=...;

dvar float+    loss;
dvar float+    z[Retailers][Products];
dvar float+    w[Retailers];
dvar boolean   x[Products];
float multiplier=0.2;

minimize loss;
subject to {
     sum(j in Products) Weight[j]*x[j]<=Capacity;
   sum(i in Retailers) Fraction[i]*RevenueMax[1]*w[i]+sum(i in Retailers, j in Products) Fraction[i]*(RevenueMax[1]-Revenue[1][j])*multiplier*Preference[i][j]*z[i][j]==loss;
 forall( i in Retailers )
    sum(j in Products) multiplier*Preference[i][j]*z[i][j]+ NoPurchase[i]*w[i]==1;
    forall(i in Retailers, j in Products)
      NoPurchase[i]*(w[i]-z[i][j])<=1-x[j];    
 forall(i in Retailers, j in Products)
      z[i][j]<=w[i];
  forall(i in Retailers, j in Products)
      (NoPurchase[i])*z[i][j]<=x[j];    
}


//int   NbProducts = ...;
//int   NbRetailers = ...;
//range Products = 1..NbProducts;
//range Retailers =1..NbRetailers;
//int Capacity = ...;
//float multiplier=0.1;
//float Fraction[Retailers] = ...;
//float Preference[Retailers][Products] = ...;
//float Revenue[Retailers][Products] = ...;
//float Weight[Products]=...;
//float RevenueMax[Retailers]=...;
//float MaximumCap[Retailers][Products]=...;
//dvar boolean    x[Products];
//dvar float+    t[Retailers][Products];
//dvar float+    t0[Retailers];
//dvar float+    w10[Retailers];
//minimize sum(i in Retailers,j in Products) Fraction[i]*(RevenueMax[1]-Revenue[1][j])*multiplier*Preference[i][j]*t[i][j]+sum(i in Retailers) RevenueMax[1]*Fraction[i]*t0[i];
// subject to {
//   forall(i in Retailers)
//     w10[i]==sum(j in Products) (multiplier*Preference[i][j]*x[j])+1;
//   forall(i in Retailers, j in Products)
//     t[i][j]*w10[i]>=x[j]^2;
//  forall(i in Retailers)
//    t0[i]*w10[i]>=1;    
//   sum(j in Products) Weight[j]*x[j]<=Capacity;   
//  forall(i in Retailers)
//      t0[i]+sum(j in Products)multiplier*Preference[i][j]*t[i][j]==1;
//  forall(i in Retailers, j in Products) 
//       t[i][j]>=x[j]*(1/(1+multiplier*MaximumCap[i][Capacity]));
//   forall(i in Retailers, j in Products) 
//        t[i][j]<=x[j];
//  forall(i in Retailers, j in Products) 
//      t[i][j]<=t0[i]+(1/(1+multiplier*MaximumCap[i][Capacity]))*x[j]-(1/(1+multiplier*MaximumCap[i][Capacity]));
//  forall(i in Retailers, j in Products) 
//      t[i][j]>=t0[i]+x[j]-1;
//Explicit Bounds
//  forall(i in Retailers)  
//      t0[i]<=1;
// forall(i in Retailers)
//      t0[i]>=1/(1+multiplier*MaximumCap[i][Capacity]);    
//  forall(i in Retailers)  
//      w10[i]>=1;
// forall(i in Retailers)
//      w10[i]<=1+multiplier*MaximumCap[i][Capacity];  
//  forall(i in Retailers)
//    sum(j in Products)t[i][j]-(1/(1+multiplier*MaximumCap[i][Capacity]))*sum(j in Products)x[j]<=(t0[i]-1/(1+multiplier*MaximumCap[i][Capacity]))*Capacity;           
//  forall(i in Retailers)
//    sum(j in Products)x[j]-sum(j in Products)t[i][j]<=Capacity*(1-t0[i]);
//}      

//int   NbProducts = ...;
//int   NbRetailers = ...;
//range Products = 1..NbProducts;
//range Retailers =1..NbRetailers;
//int Capacity = ...;
//float multiplier=0.1;
//float Fraction[Retailers] = ...;
//float Preference[Retailers][Products] = ...;
//float Revenue[Retailers][Products] = ...;
//float Weight[Products]=...;
//float RevenueMax[Retailers]=...;
//float MaximumCap[Retailers][Products]=...;
//dvar boolean    x[Products];
//dvar float+    t[Retailers][Products];
//dvar float+    t0[Retailers];
//minimize sum(i in Retailers,j in Products) Fraction[i]*(RevenueMax[1]-Revenue[1][j])*multiplier*Preference[i][j]*t[i][j]+sum(i in Retailers) RevenueMax[1]*Fraction[i]*t0[i];
// subject to {
//    sum(j in Products) Weight[j]*x[j]<=Capacity;   
//  forall(i in Retailers)
//      t0[i]+sum(j in Products)multiplier*Preference[i][j]*t[i][j]==1;
//   forall(i in Retailers, j in Products) 
//        t[i][j]>=x[j]*(1/(1+multiplier*MaximumCap[i][Capacity]));
//   forall(i in Retailers, j in Products) 
//        t[i][j]<=x[j];
//  forall(i in Retailers, j in Products) 
//      t[i][j]<=t0[i]+(1/(1+multiplier*MaximumCap[i][Capacity]))*x[j]-(1/(1+multiplier*MaximumCap[i][Capacity]));
//  forall(i in Retailers, j in Products) 
//      t[i][j]>=t0[i]+x[j]-1;
//  forall(i in Retailers)
//    sum(j in Products)t[i][j]-(1/(1+multiplier*MaximumCap[i][Capacity]))*sum(j in Products)x[j]<=(t0[i]-1/(1+multiplier*MaximumCap[i][Capacity]))*Capacity;           
//  forall(i in Retailers)
//    sum(j in Products)x[j]-sum(j in Products)t[i][j]<=Capacity*(1-t0[i]);
//}
